--[[
Версия 2.
Для символьного тега добавлен поиск по "личным" настроечным файлам
Версия 1.
Заменитель - расширитель far.GetMsg
Варианты вызова:
GetMsg( <number> ) - стандартный far.GetMsg
GetMsg( <имя_файла>, <Тег> ) - расширение.
<имя_файла> - имя языкового файла (возможно, без "языкового окончания"). Например, EMenu, EMenu.lng, esc_rus.lng
              Если расширение не указано - предполагается .lng
              Если указанный файл не найден, то ищется последовательно <имя>lang, <имя>_lang, <имя>langh, <имя>_langh, где
                lang - первые 3 символа настройки языка ФАРа; langh - то же для языка помощи
<Тег> - либо номер (тот же, что указывается в far.GetMsg), либо символьная строка, указанная в языковом файле в виде
        строки //[Тег] (см., например, %FarHome%\FarRus.lng)
-- ]]
--local BaxUtils=require"Bax.BaxUtils"
local GetMsg =
function (...)
 local Res
 if #{...} == 1 and tonumber(...) then
  return far.GetMsg(...)
 end

 local Args=...
 if type(Args) ~= 'table' then
  local ar={...}
  Args = {File=ar[1]; Tag=ar[2]}
 end
 local File, Tag=Args.File, Args.Tag
 if File and Tag then
  local function CheckFN_(s)
   if win.GetFileInfo(s) then
    File = s
    return true
   end
  end
  local function SeekTag(AFile, xTag, bCustom)
   local sRes
   local hFile=io.open(AFile, 'r')
   if hFile then
    local Found, iTag=false, -1
    for Line in hFile:lines() do
     -- Столкнулся с ситуацией, когда в начале файла - сигнатура UTF-8 (BOM). Попытка тупо удалить.
     -- Наверняка есть более корректный способ, но на выводе BOM не виден.
     if string.byte(Line,1) == 239 and    -- 0xEF
        string.byte(Line,2) == 187 and    -- 0xBB
        string.byte(Line,3) == 191 then   -- 0xBF
      Line = string.sub(Line,4)
     end
     Line = Line:match('^%s*(.-)%s*$')
     if bCustom then
      sRes = Line:match('^'..xTag..'%s*=%s*(%b"")')
      if sRes then
       sRes = sRes:sub(2,-2)
       break
      end
     elseif type(xTag) == 'string' then
      if Found then
       sRes = Line:sub(2,-2)
       break
      else
       Found = Line == '//[' .. xTag .. ']'
      end
     else
      iTag = iTag + 1
      if iTag == xTag then
       sRes = Line:sub(2,-2)
       break
      end
     end
    end
    hFile:close()
   end
   Res = sRes
   return sRes
  end -- of function SeekTag
  if not win.GetFileInfo(File) then
   local lang=(Far.GetConfig('Language.Main') or win.GetEnv('FARLANG') or 'English'):sub(1,3)
   local langh=(Far.GetConfig('Language.Help') or win.GetEnv('FARLANG') or 'English'):sub(1,3)
   local PathN, Ext=mf.fsplit(File, 7), mf.fsplit(File, 8)
   for w in PathN:gmatch('%%[^%%]+%%') do
    PathN = PathN:gsub( '%'..w..'%',win.GetEnv(w:sub(2,-2)) )
   end

   if Ext =='' then Ext = '.lng' end
   if not ( CheckFN_(PathN..Ext) or
            CheckFN_(PathN..lang..Ext) or
            CheckFN_(PathN..'_'..lang..Ext) or
            CheckFN_(PathN..langh..Ext) or
            CheckFN_(PathN..'_'..langh..Ext) or
            CheckFN_(PathN..'Eng'..Ext) or
            CheckFN_(PathN..'_'..'Eng'..Ext)
          ) then
    far.Message('Языковой файл '..PathN..'[[_]'..lang..'] не найден', 'GetMsg', nil, 'w')
   end
  end
  if win.GetFileInfo(File) then
   local File0 = File
   if type(Tag) == 'string' and
      ( ( CheckFN_(File .. '.custom') and SeekTag(File, Tag, true) ) or
        --false
        ( CheckFN_(win.GetEnv('FARHOME') .. '\\' .. mf.fsplit(File0, 12) .. '.custom') and SeekTag(File, Tag, true) ) or
        ( CheckFN_(win.GetEnv('FARPROFILE') .. '\\' .. mf.fsplit(File0, 12) .. '.custom') and SeekTag(File, Tag, true) )
      ) then
    return Res
   end
   return SeekTag(File0, Tag, false)
  end
 else
  far.Message("Требуется указать File= и Tag=", 'GetMsg', nil, 'w')
 end
 --return Res
end -- of function GetMsg
-- Делаю "личный" подраздел _G и записываю туда эту функцию
if not _G.BAX then _G.BAX = {} end
if not _G.BAX.GetMsg then
 _G.BAX.GetMsg = GetMsg
end
---------------------------------------------------------
-- Эту функцию записываю в _G, если её там не было. Для упрощения вызова...
-- Если Вам это не надо - просто уберите следующие строки
if not _G.GetMsg then
 _G.GetMsg = GetMsg
end
